
//-------
// Title
//-------
var TitleLayer = cc.Layer.extend({
	ctor:function () {
		this._super();
		
		// title
		var sp_title = new cc.Sprite(res.Title_png);
		sp_title.attr({
			x: 0,
			y: 0,
			anchorX : 0,
			anchorY : 0
		});
		this.addChild(sp_title, 1);
		
		// play button
		var btn_play = new cc.MenuItemImage(
			res.Btn_play1_png,
			res.Btn_play2_png,
			function () {
				cc.director.runScene(new GameScene());
			}, this);
		btn_play.attr({
			x: 400,
			y: 200,
			anchorX: 0.5,
			anchorY: 0.5
		});
		
		// menu
		var menu = new cc.Menu(btn_play);
		menu.x = 0;
		menu.y = 0;
		this.addChild(menu, 10);

		return true;
	}
});

var TitleScene = cc.Scene.extend({
	onEnter:function () {
		this._super();
		var layer = new TitleLayer();
		this.addChild(layer);
	}
});

//-------
// Game
//-------
var GameLayer = cc.Layer.extend({
	
	// properties
	score:0,
	scoreText:0,
	popScoreText:null,
	gameStage:[],
	icons:null,
	effectSprite:null,
	effectAnimations:null,
	selectedMark:null,
	hintMark1:null,
	hintMark2:null,
	selectedRow:-1,
	selectedCol:-1,
	tryagainButton:null,
	flgGameover:false,
	gameoverText:null,
	
	//---------
	// init
	//---------
	ctor:function () {
		this._super();
		
		// sound
		cc.audioEngine.playEffect(res.Select_snd);
		
		// bg
		var bg = new cc.Sprite(res.Bg_png);
		bg.attr({
			x: 0,
			y: 0,
			anchorX : 0,
			anchorY : 0
		});
		this.addChild(bg, 0);
		
		//------
		// UI
		//------
		// try again
		tryagainButton = new cc.MenuItemImage(
			res.Btn_tryagain1_png,
			res.Btn_tryagain2_png,
			function () {
				cc.director.runScene(new GameScene());
			}, this);
		tryagainButton.attr({
			x: 680,
			y: 520,
			anchorX: 0.5,
			anchorY: 0.5
		});
	
		// hint button
		var hintButton = new cc.MenuItemImage(
			res.Btn_hint1_png,
			res.Btn_hint2_png,
			function () {
				this.showHint();
			}, this);
		hintButton.attr({
			x: 680,
			y: 80,
			anchorX: 0.5,
			anchorY: 0.5
		});
		
		// menu
		var menu = new cc.Menu();
		menu.addChild(hintButton);
		menu.addChild(tryagainButton);
		menu.x = 0;
		menu.y = 0;
		this.addChild(menu, 1);
		
		// score
		this.scoreText = new cc.LabelTTF();
		this.scoreText.x = 680;
		this.scoreText.y = 220;
		this.scoreText.setFontSize(60);
		this.scoreText.setColor(cc.color(255,255,255,255));
		this.scoreText.enableShadow(cc.color(0,0,0,255), cc.size(3,-3), 1);
		this.scoreText.setString("0");
		this.addChild(this.scoreText);
		
		// gameover text
		this.gameoverText = new cc.LabelTTF();
		this.gameoverText.x = 400;
		this.gameoverText.y = 770;
		this.gameoverText.setFontSize(60);
		this.gameoverText.setColor(cc.color(204,0,0,255));
		this.gameoverText.enableShadow(cc.color(0,0,0,255), cc.size(1,-1), 1);
		this.gameoverText.setString("GAMEOVER");
		this.addChild(this.gameoverText, 120);
		
		// effect sprite
		var efwidth = 120;
		var efframenum = 5;
		var efamount = 6;
		this.effectSprite =  new cc.Sprite(res.Ef_create_png);
		this.effectSprite.x = -1000;
		this.effectSprite.y = -1000;
		this.effectAnimations = [];
		
		for(var efr=0; efr<efamount; efr++){
			var frames = [];
			for(var efc=0; efc<efframenum; efc++){
				frames.push(new cc.SpriteFrame(res.Ef_create_png, cc.rect(efwidth * efc, efwidth * efr, efwidth, efwidth)));
			}
			this.effectAnimations.push(new cc.Animate(new cc.Animation(frames, 0.15)));
		}
		this.addChild(this.effectSprite, 100);
		
		// pop score
		this.popScoreText = new cc.LabelTTF();
		this.popScoreText.x = -1000;
		this.popScoreText.y = -1000;
		this.popScoreText.setFontSize(50);
		this.popScoreText.setColor(cc.color(255,255,255,255));
		this.popScoreText.enableShadow(cc.color(0,0,0,255), cc.size(3,-3), 1);
		this.popScoreText.setString("0");
		this.addChild(this.popScoreText, 110);
		
		//-------------
		// touch event
		//-------------
		cc.eventManager.addListener({
			event: cc.EventListener.TOUCH_ONE_BY_ONE,
			swallowTouches:false,
			onTouchBegan:this.onTouchBegan.bind(this),
		}, this);
		
		//--------------------
		// create game stage
		//--------------------
		this.gameStage = createGameStage();
		
		// icon group
		this.icons = new cc.Node();
		this.addChild(this.icons, 10);
		
		// put icons
		for(var r=0; r<this.gameStage.length; r++){
			for(var c=0; c<this.gameStage[r].length; c++){
				var newIcon = new IconClass(this.gameStage[r][c], r, c);
				this.icons.addChild(newIcon);
			}
		}
		
		// selected icon mark
		this.selectedMark = new cc.Sprite(res.Icon_selected_png);
		this.addChild(this.selectedMark, 20);
		
		// hint icon mark
		this.hintMark1 = new cc.Sprite(res.Icon_hint_png);
		this.hintMark2 = new cc.Sprite(res.Icon_hint_png);
		this.addChild(this.hintMark1, 21);
		this.addChild(this.hintMark2, 22);
		
		// init marks
		this.deselect();
		
		// on update
		//this.scheduleUpdate();
		
		return true;
	},
	
	//-----------
	// touch
	//-----------
	onTouchBegan:function (touch, event) {
		
		// gameover
		if(this.flgGameover == true){
			return;
		}
		
		// hide hint
		this.hideHint();
		
		// check icons
		for(var i=0; i<this.icons.getChildrenCount(); i++){
			var targetIcon = this.icons.children[i];
			if(cc.rectContainsPoint(targetIcon.getBoundingBox(), touch.getLocation())){
				
				// deselect
				if(targetIcon.row == this.selectedRow && targetIcon.col == this.selectedCol){
					this.deselect();
				
				// check side by side
				}else if(
					targetIcon.row == this.selectedRow - 1 && targetIcon.col == this.selectedCol ||
					targetIcon.row == this.selectedRow && targetIcon.col == this.selectedCol -1 ||
					targetIcon.row == this.selectedRow && targetIcon.col == this.selectedCol +1 ||
					targetIcon.row == this.selectedRow + 1 && targetIcon.col == this.selectedCol
				){
					this.executeMix(targetIcon.row, targetIcon.col);
					this.deselect();
					
				// update selected item
				}else{
					this.selectedRow = targetIcon.row;
					this.selectedCol = targetIcon.col;
					this.selectedMark.x = targetIcon.x;
					this.selectedMark.y = targetIcon.y;
				}
				
				break;
			}
		}
		
		return true;
	},
	
	//---------------
	// execute mix
	//---------------
	executeMix:function(inTargetRow, inTargetCol){
		
		// get mixed
		var mixedData = getMixed(this.gameStage[this.selectedRow][this.selectedCol], this.gameStage[inTargetRow][inTargetCol]);
		var mixedItem = mixedData[0];
		var mixedScore = mixedData[1];
		var mixedEffect = mixedData[2];
		
		// if impossible
		if(mixedItem == ""){
			return;
		}
		
		// update target
		var targetIcon = this.getIconByRowCol(inTargetRow, inTargetCol);
		this.gameStage[inTargetRow][inTargetCol] = mixedItem;
		targetIcon.renew(mixedItem, inTargetRow, inTargetCol);
		
		// show effect
		this.effectSprite.x = targetIcon.x;
		this.effectSprite.y = targetIcon.y;
		this.effectSprite.runAction(this.effectAnimations[mixedEffect]);
		
		// sound
		if(mixedEffect == 0){
			cc.audioEngine.playEffect(res.Stone_snd);
		}else if(mixedEffect == 1){
			cc.audioEngine.playEffect(res.Spark_snd);
		}else if(mixedEffect == 2){
			cc.audioEngine.playEffect(res.Burn_snd);
		}else if(mixedEffect == 3){
			cc.audioEngine.playEffect(res.Slaughter_snd);
		}else if(mixedEffect == 4){
			cc.audioEngine.playEffect(res.Wood_snd);
		}else if(mixedEffect == 5){
			cc.audioEngine.playEffect(res.Upgrade_snd);
		}
		
		// score
		this.score += Number(mixedScore);
		this.scoreText.setString(this.score);
		
		// pop score
		if(mixedScore > 0){
			this.popScoreText.setString(mixedScore);
			this.popScoreText.x = targetIcon.x;
			this.popScoreText.y = targetIcon.y + 60;
			
			this.popScoreText.stopAllActions();
			var move = cc.moveTo(0.2, cc.p(this.popScoreText.x, this.popScoreText.y + 50)).easing(cc.easeIn(0.6));
			var wait = cc.delayTime(0.6);
			var hideout = cc.moveTo(0, cc.p(-1000, -1000));
			this.popScoreText.runAction(cc.sequence(move, wait, hideout));
		}
		
		// keep selected icon
		var newItemIcon = this.getIconByRowCol(this.selectedRow, this.selectedCol);
		
		// fall down
		for(var r=this.selectedRow; r>0; r--){
			this.gameStage[r][this.selectedCol] = this.gameStage[r-1][this.selectedCol];
			
			var fallIcon = this.getIconByRowCol(r-1, this.selectedCol);
			fallIcon.renew(fallIcon.id, fallIcon.row + 1, this.selectedCol);
			
			//*** simple anime ***
			fallIcon.y += ICON_WIDTH;
			fallIcon.runAction(cc.moveTo(0.2, cc.p(fallIcon.x, fallIcon.y - ICON_WIDTH)).easing(cc.easeOut(0.5)));
		}
		
		// new item
		var rnd_setup = DATA_SETUP[0];
		var rnd_id = rnd_setup[getIntRand(rnd_setup.length)];
		var newItemId = rnd_id;
		this.gameStage[0][this.selectedCol] = newItemId;
		newItemIcon.renew(newItemId, 0, this.selectedCol);
		
		//*** simple anime ***
		newItemIcon.y += ICON_WIDTH;
		newItemIcon.runAction(cc.moveTo(0.2, cc.p(newItemIcon.x, newItemIcon.y - ICON_WIDTH)).easing(cc.easeOut(0.5)));
		
		// game over
		if(this.checkMixAble() == false){
			this.deselect();
			this.flgGameover = true;
			this.gameoverText.runAction(cc.moveTo(5, cc.p(this.gameoverText.x, 570)));
		}
	},
	
	//------------
	// show hint
	//------------
	showHint:function(){
		
		this.deselect();
		
		var hintPos = this.checkMixAble();
		if(hintPos == false){
			return;
		}
		
		var targetIcon1 = this.getIconByRowCol(hintPos[0][0], hintPos[0][1]);
		var targetIcon2 = this.getIconByRowCol(hintPos[1][0], hintPos[1][1]);
		
		// set mark
		this.hintMark1.x = targetIcon1.x;
		this.hintMark1.y = targetIcon1.y;
		this.hintMark2.x = targetIcon2.x;
		this.hintMark2.y = targetIcon2.y;
		
		// sound
		cc.audioEngine.playEffect(res.Select_snd);
	},
	
	//----------------------------------------
	// check mix able
	// [[row1, col1], [row2, col2]] or false
	//----------------------------------------
	checkMixAble:function(){
		
		var rtn = false;
		
		loopRow:
		for(var r=0; r<this.gameStage.length; r++){
			for(var c=0; c<this.gameStage[r].length-1; c++){
				if(getMixed(this.gameStage[r][c], this.gameStage[r][c+1])[0] != ""){
					rtn = [[r,c], [r,c+1]];
					break loopRow;
				}
			}
		}
		
		if(rtn == false){
			loopCol:
			for(var c=0; c<this.gameStage[0].length; c++){
				for(var r=0; r<this.gameStage.length-1; r++){
					if(getMixed(this.gameStage[r][c], this.gameStage[r+1][c])[0] != ""){
						rtn = [[r+1,c], [r,c]];
						break loopCol;
					}
				}
			}
		}
		
		return rtn;
	},
	
	//----------------------
	// get icon by row col
	//----------------------
	getIconByRowCol:function(inRow, inCol){
		for(var i=0; i<this.icons.getChildrenCount(); i++){
			if(this.icons.children[i].row == inRow && this.icons.children[i].col == inCol){
				return this.icons.children[i];
			}
		}
		return null;
	},
	
	//---------------
	// deselect
	//---------------
	deselect:function(){
		this.selectedRow = -1;
		this.selectedCol = -1;
		this.selectedMark.x = -1000;
		this.selectedMark.y = -1000;
		this.hideHint();
	},
	
	//----------------
	// hide hint
	//----------------
	hideHint:function(){
		this.hintMark1.x = -1000;
		this.hintMark1.y = -1000;
		this.hintMark2.x = -1000;
		this.hintMark2.y = -1000;
	},
	
	//---------
	// update
	//---------
	update:function(){
		this._super();
		
	}
});

var GameScene = cc.Scene.extend({
	onEnter:function () {
		this._super();
		var layer = new GameLayer();
		this.addChild(layer);
	}
});

